namespace eval ::itunes {
    variable config
    variable configlist
    
    ###############################################
	# ::itunes::InitPlugin dir                    #
	# ------------------------------------------- #
	# Load proc of iTunes Plugin                  #
	###############################################	
	proc InitPlugin { dir } {
  	  ::plugins::RegisterPlugin itunes
  	  source [file join $dir itunes.tcl]
    	::plugins::RegisterEvent itunes OnConnect newname
    	::plugins::RegisterEvent itunes OnDisconnect stop
    	::plugins::RegisterEvent itunes chatmenu submenu
    	#Save in a config key where is the itunes plugin
    	::config::setKey itunespluginpath $dir
    	
    	array set ::itunes::config [list \
		nickname {} \
		second {30} \
		symbol {-} \
		stop {iTunes stopped} \
		active {1} \
    	]
    	set ::itunes::configlist [list \
   				[list bool "Add song to the nickname"  active] \
				[list str "Verify new song each ? seconds" second] \
				[list str "Nickname"  nickname] \
				[list str "Symbol betwen nick and song"  symbol] \
				[list str "Stopped message"  stop] \			  
			 ]
		#Start changing the nickname on loading
		::itunes::wait_load_newname
	}
	
    ###############################################
	# ::itunes::newname event epvar               #
	# ------------------------------------------- #
	# Main proc to change the nickname            #
	###############################################
    proc newname {event epvar} {
		variable config

		#Get all song information from ::itunes::GetSong
		set info [::itunes::GetSong]
		set software [lindex $info 0]
		set status [lindex $info 1]
		set songart [lindex $info 2]
		set path [lindex $info 3]
				
		#First add the nickname the user choosed in config to the name variable
		set name "$config(nickname)"
		#Symbol that will be betwen the nick and the song
		set separation " $config(symbol) "
		#Merge the nickname with the symbol
		append name $separation
		#Merge the nickname with the song name if a song is playing
		#Else, show the stop messsage
		if {$status != "0"} {
			append name $songart
		} else {
			append name $config(stop)
		}
		
		#Change the nickname if the user did'nt uncheck that config.
		if {$config(active)} {
			::itunes::changenick "$name"
		}
		
		#Execute the applescript to get the song name (for next loop)
		::itunes::exec_applescript

		#Take the second number we have from the plugin config and 
		#multiply by 1000 because "after" count in "ms" (1000ms=1s)
		set time [expr {int($config(second)*1000)}]
		#Reload newname proc after this time (loop)
		after $time ::itunes::newname 0 0
	
    }
    ###############################################
	# ::itunes::changenick name                   #
	# ------------------------------------------- #
	# Protocol to change the nickname             #
	###############################################
    proc changenick {name} {
    	set email [::config::getKey login]
   		::MSN::changeName $email $name
    }
    
    ###############################################
	# ::itunes::GetSong                           #
	# ------------------------------------------- #
	# Read the file actualsong and return result  #
	###############################################
    proc GetSong {} {
		#Osascript is a command on  Mac OS X to get result from an AppleScript
		#catch {exec osascript plugins/applescript/display_and_send.scpt &} return
		
		#Find the file to read
		set file "~/Library/Application\ Support/amsn/plugins/actualsong"
		#Verify that the file exist
		if {![file exist $file]} {return 0}
		#Open in "read" permission the file (SongIngo)
		set gets [open $file r]
			
		#Get the 4 first lines
		set line1 [gets $gets]
		set line2 [gets $gets]
		set line3 [gets $gets]
		set line4 [gets $gets]
		#Include theses 4 lines to $return
		lappend return $line1
		lappend return $line2
		lappend return $line3
		lappend return $line4
		
	
		#Close the file
		close $gets
		
		return $return
	
	}
	
	################################################
	# ::itunes::submenu event epvar                #
	# -------------------------------------------  #
    # Create submenu inside chatwindow 	           #
    # Control-click (right click) to activate	   #
	################################################	
	proc submenu { event evpar } {
		upvar 2 evPar newvar
		variable config
		#Create submenu iTunes
		$newvar(window_name).copy add cascade -label "iTunes" -menu $newvar(window_name).copy.itunes
		menu $newvar(window_name).copy.itunes -tearoff 0 -type normal
		
		#Add labels in the menu 
		$newvar(window_name).copy.itunes add command -label [trans xmmscurrent] \
			-command "::itunes::exec_applescript ; \
				after 7000 ::itunes::menucommand $newvar(window_name) 1"
		$newvar(window_name).copy.itunes add command -label [trans xmmssend] \
			-command "::itunes::exec_applescript ; \
				after 7000 ::itunes::menucommand $newvar(window_name) 2"

	}
	
	################################################
	# ::itunes::exec_applescript                   #
	# -------------------------------------------  #
    # Execute the applescript to get actual song   #
    # Osascript: Command to execute AppleScript    #
    # Find where's the plugin directory(by getKey) #
	################################################	
	proc exec_applescript {} {
		set dir [::config::getKey itunespluginpath]
		catch {exec osascript $dir/display_and_send.scpt &}
	}

	################################################
	# ::itunes::menucommand window_name action     #
	# -------------------------------------------  #
    # Command from the submenu      	           #
    # Display the current song in a message (1)    #
    # Or send the current song playing (2)         #
	################################################
	proc menucommand {window_name action} {
	
		#Get all song information from ::itunes::GetSong
		set info [::itunes::GetSong]
		set software [lindex $info 0]
		set status [lindex $info 1]
		set songart [lindex $info 2]
		set path [lindex $info 3]
		set pathstart [string range $path 0 0]
		
		#Status 0 means that iTunes is stopped
		if {$status == "0"} {return 0}
		
		switch -- $action {
			1 {
				# - Display current song. (1)
				::amsn::MessageSend $window_name 0 "[trans playing $songart]"
				return 0
			}
			2 {
			#Verify that we get a real path, if not, don't send the file
				if {${pathstart} == {/} } { 
					::amsn::FileTransferSend $window_name $path
					return 0
				} else {
					set msg "You can't send this file";
					::amsn::messageBox $msg ok error ""
					return 0
				}
			}
		}
	
	}
    
    ###############################################
	# ::itunes::wait_load_newname                 #
	# ------------------------------------------- #
	# Wait 1 second before starting changing     #
	# the nickname. The plugin only use that proc #
	# when we load the plugin. To be sure that    #
	# the init proc is over                       #
	############################################### 
    proc wait_load_newname {} {
   		after 1000 ::itunes::load_newname 0 0
    }
    
    ###############################################
	# ::itunes::load_newname  event epvar         #
	# ------------------------------------------- #
	# Start changing the nick                     #
	# If we load the plugin while we were already #
	# connected, start changing the nick, else do #
	# nothing                                     #
	###############################################
    proc load_newname {event epvar} {
    
   		 #If we are online, start the loop
   		if {[sb get ns stat] == "o" } {
    		::itunes::newname 0 0
  		}
    }
    

    ###############################################
	# ::itunes::stop event epvar                  #
	# ------------------------------------------- #
	# Stop changing the nickname                  #
	# Happen when we disconnect or when we unload #
	# This is the Deinit proc                     #
	###############################################
    proc stop {{event 0} {epvar 0}} {
    	variable config
		after cancel ::itunes::newname 0 0
		#Remove the song from the nick if we are online
		if {[sb get ns stat] == "o" } {
	   		::itunes::changenick "$config(nickname)"
	   	}
    }
}